#include <windows.h>
#include "config.h"



#define A_RO 0x1
#define A_H	0x2
#define A_S	0x4
#define A_V	0x8
#define A_D	0x10
#define A_A	0x20

#define A_MOD		(A_RO+A_H+A_S+A_A)
#define ISSLASH(c)	((c)=='\\' || (c)=='/')


extern __int64 FT_EPOCH;
extern __int64 FT_TICKS;

static int is_unc(const CHAR *path)
{	CHAR *p=NULL;

	if((strlen(path) >= 5) && ISSLASH(path[0]) && ISSLASH(path[1]))
	{	(const CHAR *)p=path+2;
		while(* ++ p)
			if(ISSLASH(*p))
				break ;
		if(*p && p[1])
		{	while(* ++ p)
				if(ISSLASH(*p))
				break ;
			if (!*p || !p[1])
				return 1;
		}
	}
	return 0;
}

unsigned short dtoxmode(int attr, const CHAR *path)
{	unsigned short umode;
	unsigned dmode;
	const CHAR *p;

	dmode=attr & 0xff;
	if((p=path)[1]==':')
            p+=2;
	umode=(unsigned short)(((ISSLASH(*p) && !p[1]) || (dmode & A_D) || !*p)
		  ? _S_IFDIR|_S_IEXEC : _S_IFREG);
	umode |= (dmode & A_RO) ? _S_IREAD : (_S_IREAD|_S_IWRITE);
	if(p=strrchr(path, '.'))
	{	if(p[1] && (p[1]=='e' || p[1]=='c' || p[1]=='b') &&
		   p[2] && (p[2]=='x' || p[2]=='m' || p[2]=='a' || p[2]=='o') &&
		   p[3] && (p[3]=='e' || p[3]=='d' || p[3]=='t' || p[3]=='m'))
		   umode |= _S_IEXEC;
	}
	umode |= (umode & 0700) >> 3;
	umode |= (umode & 0700) >> 6;
	return(umode);
}




int stat(const CHAR *path, struct stat *st)
{	WCHAR wzPath[MAX_PATH+1];
	WIN32_FIND_DATA wfData;
	HANDLE hFile;

    if(strchr(path,'?') || strchr(path, '*'))
		return(-1);

	MultiByteToWideChar(CP_ACP, 0, path, -1, wzPath, MAX_PATH+1);
	hFile=FindFirstFile(wzPath, &wfData);

	if(hFile==INVALID_HANDLE_VALUE)
	{	return -1;
	}
	else
	{	st->st_mtime=(time_t)((*(__int64*)&wfData.ftLastWriteTime-FT_EPOCH)/FT_TICKS);

		if(wfData.ftLastAccessTime.dwLowDateTime || wfData.ftLastAccessTime.dwHighDateTime)
		{	
			st->st_atime=(time_t)((*(__int64*)&wfData.ftLastAccessTime-FT_EPOCH)/FT_TICKS);
		}
		else	st->st_atime=st->st_mtime ;

		if(wfData.ftCreationTime.dwLowDateTime || wfData.ftCreationTime.dwHighDateTime )
		{	
			st->st_ctime=(time_t)((*(__int64*)&wfData.ftCreationTime-FT_EPOCH)/FT_TICKS);

		}
		else	st->st_ctime=st->st_mtime ;

		FindClose(hFile);
	}
	st->st_mode=dtoxmode(wfData.dwFileAttributes, path);
	st->st_nlink=1;
	st->st_size=(_off_t)wfData.nFileSizeLow;
	st->st_uid=st->st_gid=st->st_ino=0;
	st->st_rdev=st->st_dev=(_dev_t)0;
	return 0;
}

int fstat(int handle, struct stat *st)
{	
	BY_HANDLE_FILE_INFORMATION wfData;
	memset(&wfData, 0, sizeof(wfData));
	if (!GetFileInformationByHandle((HANDLE)handle, &wfData))
		return -1;

	st->st_mtime=(time_t)((*(__int64*)&wfData.ftLastWriteTime-FT_EPOCH)/FT_TICKS);

	if(wfData.ftLastAccessTime.dwLowDateTime || wfData.ftLastAccessTime.dwHighDateTime)
	{	
		st->st_atime=(time_t)((*(__int64*)&wfData.ftLastAccessTime-FT_EPOCH)/FT_TICKS);
	}
	else	st->st_atime=st->st_mtime ;

	if(wfData.ftCreationTime.dwLowDateTime || wfData.ftCreationTime.dwHighDateTime )
	{	
		st->st_ctime=(time_t)((*(__int64*)&wfData.ftCreationTime-FT_EPOCH)/FT_TICKS);
	}
	else	st->st_ctime=st->st_mtime ;

//	st->st_mode=dtoxmode(wfData.dwFileAttributes, path);
	st->st_nlink=1;
	st->st_size=(_off_t)wfData.nFileSizeLow;
	st->st_uid=st->st_gid=st->st_ino=0;
	st->st_rdev=st->st_dev=(_dev_t)0;
	return 0;
}

