/********************************************************************

 import_nt.c 

  Win32 specific import code.

*/

#include "Python.h"
#include "osdefs.h"
#ifndef DONT_HAVE_ASSERT_H
#include <assert.h>
#endif
#include <windows.h>
#include "importdl.h"
#include "malloc.h" /* for alloca */

#include  <tchar.h>

/* a string loaded from the DLL at startup */
extern const TCHAR *PyWin_DLLVersionString;

FILE *PyWin_FindRegisteredModule(const char *moduleName,
				 struct filedescr **ppFileDesc,
				 char *pathBuf,
				 int pathLen)
{
	TCHAR *moduleKey;
	const TCHAR keyPrefix[] = _T("Software\\Python\\PythonCore\\");
	const TCHAR keySuffix[] = _T("\\Modules\\");
#ifdef _DEBUG
	/* In debugging builds, we _must_ have the debug version
	 * registered.
	 */
	const TCHAR debugString[] = _T("\\Debug");
#else
	const TCHAR debugString[] = _T("");
#endif
	struct filedescr *fdp = NULL;
	FILE *fp = NULL;
	HKEY keyBase = HKEY_CURRENT_USER;
	int modNameSize;
	DWORD lptype;
	long regStat;

	/* Calculate the size for the sprintf buffer.
	 * Get the size of the chars only, plus 1 NULL.
	 */
	size_t bufSize = (sizeof(keyPrefix)/sizeof(TCHAR))-1 +
	                 _tcslen(PyWin_DLLVersionString) +
	                 (sizeof(keySuffix)/sizeof(TCHAR)) +
	                 strlen(moduleName) +
	                 (sizeof(debugString)/sizeof(TCHAR)) - 1;
	/* alloca == no free required, but memory only local to fn,
	 * also no heap fragmentation!
	 */
#ifdef	UNICODE
	TCHAR *tPathBuf = (TCHAR *)malloc(pathLen * sizeof(TCHAR));
#else
	TCHAR *tPathBuf = pathBuf;
#endif

#ifndef	MS_WINCE
	moduleKey = alloca(bufSize); 
	PyOS_snprintf(moduleKey, bufSize,
		      "Software\\Python\\PythonCore\\%s\\Modules\\%s%s",
		      PyWin_DLLVersionString, moduleName, debugString);
#else
	// I think this should be the version used on all Windows.. But I'm not certain
	// The above code block is new in 2.2, so not sure which is the best way.
	moduleKey = alloca(bufSize*sizeof(TCHAR)); 
	wsprintf(moduleKey,
		_T("Software\\Python\\PythonCore\\%s\\Modules\\%hs%s"),
	        PyWin_DLLVersionString, moduleName, debugString);
	// note, %hs means char * even if unicode enabled
#endif
	modNameSize = pathLen;
	regStat = RegQueryValueEx(keyBase, moduleKey,NULL,&lptype,(char *) tPathBuf, &modNameSize);
	if (regStat != ERROR_SUCCESS) {
		/* No user setting - lookup in machine settings */
		keyBase = HKEY_LOCAL_MACHINE;
		/* be anal - failure may have reset size param */
		modNameSize = pathLen;
		regStat = RegQueryValueEx(keyBase, moduleKey, NULL,&lptype,
		                        (char *) tPathBuf, &modNameSize);

		if (regStat != ERROR_SUCCESS)
			goto done;
//			return NULL;
	}
#ifdef	UNICODE
	WideCharToMultiByte(CP_ACP,0,tPathBuf,modNameSize,pathBuf,modNameSize,NULL,NULL);
#endif
	/* use the file extension to locate the type entry. */
	for (fdp = _PyImport_Filetab; fdp->suffix != NULL; fdp++) {
		size_t extLen = strlen(fdp->suffix);
		assert(modNameSize >= 0); /* else cast to size_t is wrong */
#ifdef	MS_WINCE	// could be case sensitive on windows too
		if ((size_t)modNameSize > extLen &&
		    strncmp(pathBuf + ((size_t)modNameSize-extLen-1),
		             fdp->suffix,
		             extLen) == 0)

#else
		if ((size_t)modNameSize > extLen &&
		    strnicmp(pathBuf + ((size_t)modNameSize-extLen-1),
		             fdp->suffix,
		             extLen) == 0)
#endif
			break;
	}
	if (fdp->suffix == NULL)
		goto done;
//		return NULL;
	fp = fopen(pathBuf, fdp->mode);
	if (fp != NULL)
		*ppFileDesc = fdp;

done:;
#ifdef	UNICODE
	if(tPathBuf) free(tPathBuf);
#endif
	return fp;
}
